@@ -22,14 +22,44 @@
 #define	DISPLAY_HEX	/* the "hex" display mode */
 #define	PROTOCOL_HTTP	/* the "http" network protocol */
 #define	PROTOCOL_FTP	/* the "ftp" network protocol */
-#define FEATURE_SHOWTAG	/* the "showtag" option */
-#define FEATURE_LPR	/* the ":lpr" command */
 #define	FEATURE_ALIAS	/* the ":alias" command */
-#define	FEATURE_MKEXRC	/* the ":mkexrc" command */
-#define FEATURE_COMPLETE /* filename completion */
-#undef	FEATURE_RAM	/* store edit buffers in RAM if "-f ram" */
+#define	FEATURE_ARRAY	/* subscripts in expressions */
+#define	FEATURE_AUTOCMD	/* the :autocmd command */
+#define	FEATURE_BACKTICK /* the `program` notation in file names */
+#define	FEATURE_BROWSE	/* the :browse and :sbrowse commands */
+#define	FEATURE_CACHEDESC /* store syntax/markup descriptions in RAM */
+#define FEATURE_CALC	/* the calculator, and all cmds that use it */
+#define	FEATURE_COMPLETE /* filename completion */
+#define	FEATURE_FOLD	/* the :fold and :unfold commands */
+#define	FEATURE_G	/* most of the visual 'g' commands */
+#define	FEATURE_HLOBJECT /* the hlobject and hllayers options */
+#define	FEATURE_HLSEARCH  /* the hlsearch option */
+#ifdef GUI_WIN32
+#define FEATURE_IMAGE	/* gui can use background image */
+#endif
+#define	FEATURE_INCSEARCH /* the incsearch option */
 #define	FEATURE_LITRE	/* faster searches for literal strings */
-#define FEATURE_BACKTICK /* the `program` notation in file names */
+#define	FEATURE_LPR	/* the ":lpr" command */
+#define	FEATURE_MAKE	/* the :make and :cc commands */
+#define	FEATURE_MAPDB	/* the map debugger */
+#define	FEATURE_MISC	/* lots of little things */
+#define	FEATURE_MKEXRC	/* the ":mkexrc" command */
+#define	FEATURE_NORMAL	/* vim-style :normal command */
+#define	FEATURE_PROTO	/* using aliases to add new protocols */
+#undef	FEATURE_RAM	/* store edit buffers in RAM if "-f ram" */
+#undef	FEATURE_RCSID	/* include RCS Id strings for all source files */
+#define	FEATURE_REGION	/* the :region command */
+#define	FEATURE_SHOWTAG	/* the "showtag" option */
+#define	FEATURE_SMARTARGS /* show arguments when inputting a function call */	
+#define	FEATURE_STDIN	/* ability to use "-" as a file, to read stdin */
+#define	FEATURE_SPELL	/* spell checking */
+#define	FEATURE_SPLIT	/* :split and friends */
+#define	FEATURE_TAGS	/* :tag command -- undef'ing will break ref & ctags */
+#define	FEATURE_TEXTOBJ	/* text objects */
+#define	FEATURE_V	/* the v/V/^V marking commands */
+#undef	FEATURE_XFT	/* antialiased fonts in X11 -- not useful for Windows */
+#define FEATURE_LISTCHARS /* the listchars option */
+#define	FEATURE_EQUALTILDE /* :let option =~ excmdline */
 
 /* The following provide custom implementation of some common functions which
  * are either missing or poorly implemented on some systems.
@@ -40,7 +70,7 @@
 #if !defined(GUI_WIN32)
 #define NEED_TGETENT	/* uses tinytcap instead of termcap library */
 #endif
-#define NEED_CTYPE
+#define NEED_CTYPE	/* use elvis' digraph-sensitive version of <ctype.h> */
 
 /* The following controls debugging features */
 /* (NDEBUG is handled by "e2.mak" - defined for WinRel, undef'ed for WinDebug */
@@ -49,6 +79,7 @@
 #undef	DEBUG_SESSION	/* define to debug the block cache; undef to disable */
 #undef	DEBUG_EVENT	/* define to trace events; undef to disable */
 #undef	DEBUG_MARKUP	/* define to debug markup display modes */
+#undef	DEBUG_REGEXP	/* define to debug regular expressions */
 
 /* The following terminal descriptions are used in tinytcap.c */
 #undef	TERM_925	/* 925, and many other non-ANSI terminals */
