@@ -16,15 +16,15 @@ END_EXTERNC
 
 #else
 
-# define safealloc(qty, size)	_safealloc(__FILE__, __LINE__, False, qty, size)
-# define safekept(qty, size)	_safealloc(__FILE__, __LINE__, True, qty, size)
+# define safealloc(qty, size)	_safealloc(__FILE__, __LINE__, ElvFalse, qty, size)
+# define safekept(qty, size)	_safealloc(__FILE__, __LINE__, ElvTrue, qty, size)
 # define safefree(ptr)		_safefree(__FILE__, __LINE__, ptr)
-# define safedup(str)		_safedup(__FILE__, __LINE__, False, str)
-# define safekdup(str)		_safedup(__FILE__, __LINE__, True, str)
+# define safedup(str)		_safedup(__FILE__, __LINE__, ElvFalse, str)
+# define safekdup(str)		_safedup(__FILE__, __LINE__, ElvTrue, str)
 BEGIN_EXTERNC
-extern void *_safealloc P_((char *file, int line, BOOLEAN kept, int qty, size_t size));
+extern void *_safealloc P_((char *file, int line, ELVBOOL kept, int qty, size_t size));
 extern void _safefree P_((char *file, int line, void *ptr));
-extern char *_safedup P_((char *file, int line, BOOLEAN kept, char *str));
+extern char *_safedup P_((char *file, int line, ELVBOOL kept, char *str));
 extern void safeterm P_((void));
 extern void safeinspect P_((void));
 END_EXTERNC
